export declare const brackets: RegExp;
export declare const whitespace: RegExp;
export declare const reservedOps: RegExp;
export declare const trailingWhitespace: RegExp;
declare type BasicType = {
    type: 'num';
} | {
    type: 'str';
} | {
    type: 'bool';
};
export declare namespace Schema {
    type Common = {
        multi?: boolean;
        req?: boolean;
    };
    type WithBlock = {
        block?: Block;
    };
    type Enum = {
        type: 'enum';
        options: string[];
    };
    type Hash = {
        type: 'hash';
        schema: Record<string, t>;
        req?: boolean;
    };
    type Array = {
        type: 'array';
        schema: Record<string, t>;
        req?: boolean;
    };
    type Block = Hash | Array;
    export type TupleMemberType = BasicType | Enum;
    export type BasicTypeT = BasicType & Common & WithBlock;
    export type BlockT = Block & Common;
    export type EnumT = Enum & Common;
    export type KvT = {
        type: 'kv';
    } & Common;
    export type ListT = {
        type: 'list';
        of: t;
    } & Common;
    export type TupleT = {
        type: 'tuple';
        schema: TupleMemberType[];
    } & Common & WithBlock;
    export type t = BasicTypeT | BlockT | EnumT | KvT | ListT | TupleT;
    export {};
}
export declare const validTypes: string[];
export declare const LISTABLE_TYPES: string[];
export declare const BLOCKABLE_TYPES: string[];
export declare function basicTypeFromName(name: string): BasicType | null;
declare type ZamlErrorType = 'syntax-error' | 'author-error' | 'user-error' | 'unexpected-error';
export declare class ZamlError extends Error {
    type: ZamlErrorType;
    pos: Pos | null;
    static syntax(pos: Pos, char: string, more?: string): ZamlError;
    constructor(type: ZamlErrorType, pos: Pos | null, message: string);
    addSource(source: string): void;
}
export declare function isObj(x: any): boolean;
export declare function unexp(char: string, more?: string): string;
export declare function dedent(string: string): string;
export declare class Pos {
    line: number;
    col: number;
    i: number;
    constructor(line?: number, col?: number, i?: number);
    push(source: string): this;
    newcol(): this;
    newline(windowsLine?: boolean): void;
    copy(): Pos;
    goto(target: Pos): void;
    skipSpace(source: string): boolean;
    skipNewline(source: string): boolean;
    skipWhitespace(source: string): boolean;
}
export {};
